@@ -5,6 +5,7 @@ serialization via the pickle module.
 import textwrap
 
 import py
+
 import pytest
 
 pythonlist = ["python2.7", "python3.4", "python3.5"]
@@ -32,7 +33,7 @@ class Python(object):
         dumpfile = self.picklefile.dirpath("dump.py")
         dumpfile.write(
             textwrap.dedent(
-                """\
+                r"""
                 import pickle
                 f = open({!r}, 'wb')
                 s = pickle.dump({!r}, f, protocol=2)
@@ -48,7 +49,7 @@ class Python(object):
         loadfile = self.picklefile.dirpath("load.py")
         loadfile.write(
             textwrap.dedent(
-                """\
+                r"""
                 import pickle
                 f = open({!r}, 'rb')
                 obj = pickle.load(f)
