@@ -1,24 +1,25 @@
 # -*- coding: utf-8 -*-
-from __future__ import absolute_import, division, print_function
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
 
 import operator
 import os
 import sys
 import textwrap
-import _pytest
-import py
-import pytest
-import six
-from _pytest._code.code import (
-    ExceptionInfo,
-    FormattedExcinfo,
-    ReprExceptionInfo,
-    ExceptionChainRepr,
-)
-from six.moves import queue
 
+import py
+import six
+from six.moves import queue
 from test_source import astonly
 
+import _pytest
+import pytest
+from _pytest._code.code import ExceptionChainRepr
+from _pytest._code.code import ExceptionInfo
+from _pytest._code.code import FormattedExcinfo
+from _pytest._code.code import ReprExceptionInfo
+
 try:
     import importlib
 except ImportError:
@@ -31,6 +32,14 @@ failsonjython = pytest.mark.xfail("sys.platform.startswith('java')")
 pytest_version_info = tuple(map(int, pytest.__version__.split(".")[:3]))
 
 
+@pytest.fixture
+def limited_recursion_depth():
+    before = sys.getrecursionlimit()
+    sys.setrecursionlimit(150)
+    yield
+    sys.setrecursionlimit(before)
+
+
 class TWMock(object):
     WRITE = object()
 
@@ -239,7 +248,7 @@ class TestTraceback_f_g_h(object):
                 raise RuntimeError("hello")
             f(n - 1)
 
-        excinfo = pytest.raises(RuntimeError, f, 100)
+        excinfo = pytest.raises(RuntimeError, f, 25)
         monkeypatch.delattr(excinfo.traceback.__class__, "recursionindex")
         repr = excinfo.getrepr()
         assert "RuntimeError: hello" in str(repr.reprcrash)
@@ -1176,20 +1185,28 @@ raise ValueError()
         assert tw.lines[47] == ":15: AttributeError"
 
     @pytest.mark.skipif("sys.version_info[0] < 3")
-    def test_exc_repr_with_raise_from_none_chain_suppression(self, importasmod):
+    @pytest.mark.parametrize("mode", ["from_none", "explicit_suppress"])
+    def test_exc_repr_chain_suppression(self, importasmod, mode):
+        """Check that exc repr does not show chained exceptions in Python 3.
+        - When the exception is raised with "from None"
+        - Explicitly suppressed with "chain=False" to ExceptionInfo.getrepr().
+        """
+        raise_suffix = " from None" if mode == "from_none" else ""
         mod = importasmod(
             """
             def f():
                 try:
                     g()
                 except Exception:
-                    raise AttributeError() from None
+                    raise AttributeError(){raise_suffix}
             def g():
                 raise ValueError()
-        """
+        """.format(
+                raise_suffix=raise_suffix
+            )
         )
         excinfo = pytest.raises(AttributeError, mod.f)
-        r = excinfo.getrepr(style="long")
+        r = excinfo.getrepr(style="long", chain=mode != "explicit_suppress")
         tw = TWMock()
         r.toterminal(tw)
         for line in tw.lines:
@@ -1199,7 +1216,9 @@ raise ValueError()
         assert tw.lines[2] == "        try:"
         assert tw.lines[3] == "            g()"
         assert tw.lines[4] == "        except Exception:"
-        assert tw.lines[5] == ">           raise AttributeError() from None"
+        assert tw.lines[5] == ">           raise AttributeError(){}".format(
+            raise_suffix
+        )
         assert tw.lines[6] == "E           AttributeError"
         assert tw.lines[7] == ""
         line = tw.get_write_msg(8)
@@ -1341,11 +1360,13 @@ def test_cwd_deleted(testdir):
     assert "INTERNALERROR" not in result.stdout.str() + result.stderr.str()
 
 
+@pytest.mark.usefixtures("limited_recursion_depth")
 def test_exception_repr_extraction_error_on_recursion():
     """
     Ensure we can properly detect a recursion error even
     if some locals raise error on comparison (#2459).
     """
+    from _pytest.pytester import LineMatcher
 
     class numpy_like(object):
         def __eq__(self, other):
@@ -1361,40 +1382,30 @@ def test_exception_repr_extraction_error_on_recursion():
     def b(x):
         return a(numpy_like())
 
-    try:
+    with pytest.raises(RuntimeError) as excinfo:
         a(numpy_like())
-    except:  # noqa
-        from _pytest._code.code import ExceptionInfo
-        from _pytest.pytester import LineMatcher
 
-        exc_info = ExceptionInfo()
-
-        matcher = LineMatcher(str(exc_info.getrepr()).splitlines())
-        matcher.fnmatch_lines(
-            [
-                "!!! Recursion error detected, but an error occurred locating the origin of recursion.",
-                "*The following exception happened*",
-                "*ValueError: The truth value of an array*",
-            ]
-        )
+    matcher = LineMatcher(str(excinfo.getrepr()).splitlines())
+    matcher.fnmatch_lines(
+        [
+            "!!! Recursion error detected, but an error occurred locating the origin of recursion.",
+            "*The following exception happened*",
+            "*ValueError: The truth value of an array*",
+        ]
+    )
 
 
+@pytest.mark.usefixtures("limited_recursion_depth")
 def test_no_recursion_index_on_recursion_error():
     """
     Ensure that we don't break in case we can't find the recursion index
     during a recursion error (#2486).
     """
-    try:
 
-        class RecursionDepthError(object):
-            def __getattr__(self, attr):
-                return getattr(self, "_" + attr)
+    class RecursionDepthError(object):
+        def __getattr__(self, attr):
+            return getattr(self, "_" + attr)
 
+    with pytest.raises(RuntimeError) as excinfo:
         RecursionDepthError().trigger
-    except:  # noqa
-        from _pytest._code.code import ExceptionInfo
-
-        exc_info = ExceptionInfo()
-        assert "maximum recursion" in str(exc_info.getrepr())
-    else:
-        assert 0
+    assert "maximum recursion" in str(excinfo.getrepr())
