@@ -1,6 +1,10 @@
-from __future__ import absolute_import, division, print_function
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+
 import subprocess
 import sys
+
 import pytest
 
 # test for _argcomplete but not specific for any application
