@@ -1,22 +1,26 @@
 # -*- coding: utf-8 -*-
-from __future__ import absolute_import, division, print_function
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
 
-# note: py.io capture tests where copied from
-# pylib 1.4.20.dev2 (rev 13d9af95547e)
-import pickle
+import contextlib
 import os
+import pickle
 import sys
 import textwrap
 from io import UnsupportedOperation
 
 import py
-import pytest
-import contextlib
 from six import text_type
+
+import pytest
 from _pytest import capture
 from _pytest.capture import CaptureManager
 from _pytest.main import EXIT_NOTESTSCOLLECTED
 
+# note: py.io capture tests where copied from
+# pylib 1.4.20.dev2 (rev 13d9af95547e)
+
 
 needsosdup = pytest.mark.skipif(
     not hasattr(os, "dup"), reason="test needs os.dup, not available on this platform"
